#!/usr/bin/env python
# encoding: utf-8
# ----------------------------------------------------------------------------

"""
The "engine room" of django-yubin mailer.

Methods here actually handle the sending of queued messages.

"""
from django.core.mail import get_connection
from django.utils.encoding import force_bytes, smart_str
from django.utils.timezone import now
from django_yubin import constants, models, settings
from lockfile import FileLock, AlreadyLocked, LockTimeout
from socket import error as SocketError
import logging
import os
import smtplib
import tempfile
import time

from .iter_utils import peek


logger = logging.getLogger('django_yubin.engine')


LOCK_PATH = settings.LOCK_PATH or os.path.join(tempfile.gettempdir(),
                                               'send_mail')


def _message_queue(block_size, message_limit):
    """
    A generator which iterates queued messages in blocks so that new
    prioritised messages can be inserted during iteration of a large number of
    queued messages.

    To avoid an infinite loop, yielded messages *must* be deleted or deferred.

    """
    def get_block():
        queue = models.QueuedMessage.objects.non_deferred().select_related()
        if block_size:
            if message_limit:
                limit = min(block_size, message_limit)
            else:
                limit = block_size
            queue = queue[:limit]
        elif message_limit:
            queue = queue[:message_limit]
        return queue

    queue = get_block()
    messages_processed = 0
    while queue:
        for message in queue:
            yield message
            messages_processed += 1
        if message_limit and messages_processed >= message_limit:
            break
        queue = get_block()


def send_all(block_size=500, backend=None, messages=None, message_limit=0):
    """
    Send all non-deferred messages in the queue.

    A lock file is used to ensure that this process can not be started again
    while it is already running.

    The ``block_size`` argument allows for queued messages to be iterated in
    blocks, allowing new prioritised messages to be inserted during iteration
    of a large number of queued messages.

    """
    lock = FileLock(LOCK_PATH)

    logger.debug("Acquiring lock...")
    try:
        # lockfile has a bug dealing with a negative LOCK_WAIT_TIMEOUT (which
        # is the default if it's not provided) systems which use a LinkFileLock
        # so ensure that it is never a negative number.
        timeout = settings.LOCK_WAIT_TIMEOUT
        if not timeout or timeout < 0:
            timeout = 0
        lock.acquire(timeout)
    except AlreadyLocked:
        logger.debug("Lock already in place. Exiting.")
        return
    except LockTimeout:
        logger.debug("Waiting for the lock timed out. Exiting.")
        return
    logger.debug("Lock acquired.")

    start_time = time.time()

    sent = deferred = skipped = 0

    try:
        messages_queue = messages or _message_queue(block_size, message_limit)
        first_message, messages_list = peek(messages_queue)

        if not first_message:
            logger.info('No messages in queue.')
            return

        connection = get_connection(backend=backend)
        blacklist = models.Blacklist.objects.values_list('email', flat=True)
        connection.open()
        for message in messages_list:
            result = send_queued_message(message, smtp_connection=connection,
                                         blacklist=blacklist)
            if result == constants.RESULT_SENT:
                sent += 1
            elif result == constants.RESULT_FAILED:
                deferred += 1
            elif result == constants.RESULT_SKIPPED:
                skipped += 1
        connection.close()
    finally:
        logger.debug("Releasing lock...")
        lock.release()
        logger.debug("Lock released.")

    logger.debug("")
    if deferred or skipped:
        log = logger.warning
    else:
        log = logger.info
    log("%s sent, %s deferred, %s skipped." % (sent, deferred, skipped))
    logger.debug("Completed in %.2f seconds." % (time.time() - start_time))


def send_loop(empty_queue_sleep=None):
    """
    Loop indefinitely, checking queue at intervals and sending and queued
    messages.

    The interval (in seconds) can be provided as the ``empty_queue_sleep``
    argument. The default is attempted to be retrieved from the
    ``MAILER_EMPTY_QUEUE_SLEEP`` setting (or if not set, 30s is used).

    """
    empty_queue_sleep = empty_queue_sleep or settings.EMPTY_QUEUE_SLEEP
    while True:
        while not models.QueuedMessage.objects.all():
            logger.debug("Sleeping for %s seconds before checking queue "
                         "again." % empty_queue_sleep)
            time.sleep(empty_queue_sleep)
        send_all()


def send_queued_message(queued_message, smtp_connection=None, blacklist=None,
                        log=True):
    """
    Send a queued message, returning a response code as to the action taken.

    The response codes can be found in ``django_yubin.constants``. The
    response will be either ``RESULT_FAILED`` for a deferred message,
    ``RESULT_SENT`` for a successful sent message or ``RESULT_SKIPPED`` for
    blacklisted email or if ``settings.PAUSE_SEND`` is True.

    To allow optimizations if multiple messages are to be sent, an SMTP
    connection can be provided and a list of blacklisted email addresses.
    Otherwise an SMTP connection will be opened to send this message and the
    email recipient address checked against the ``Blacklist`` table.

    If the message recipient is blacklisted, the message will be removed from
    the queue without being sent. Otherwise, the message is attempted to be
    sent with an SMTP failure resulting in the message being flagged as
    deferred so it can be tried again later.

    By default, a log is created as to the action. Either way, the original
    message is not deleted.

    """
    message = queued_message.message
    if smtp_connection is None:
        smtp_connection = get_connection()
    opened_connection = False

    if blacklist is None:
        blacklisted = models.Blacklist.objects.filter(email=message.to_address)
    else:
        blacklisted = message.to_address in blacklist

    log_message = ''
    if blacklisted:
        logger.info("Not sending to blacklisted email: %s" %
                    message.to_address.encode("utf-8"))
        queued_message.delete()
        result = constants.RESULT_SKIPPED
    elif settings.PAUSE_SEND:
        logger.info("Sending is paused, deferring email.")
        queued_message.defer()
        result = constants.RESULT_SKIPPED
    else:
        try:
            logger.info("Sending message to %s: %s" %
                        (message.to_address.encode("utf-8"),
                         message.subject.encode("utf-8")))
            opened_connection = smtp_connection.open()
            try:
                smtp_connection.connection.sendmail(
                    message.from_address,
                    [message.to_address],
                    smart_str(message.encoded_message).encode('utf-8'))
            except UnicodeDecodeError:
                smtp_connection.connection.sendmail(
                    message.from_address,
                    [message.to_address],
                    force_bytes(message.encoded_message))
            queued_message.message.date_sent = now()
            queued_message.message.save()
            queued_message.delete()
            result = constants.RESULT_SENT
        except (SocketError, smtplib.SMTPSenderRefused,
                smtplib.SMTPRecipientsRefused, smtplib.SMTPAuthenticationError,
                UnicodeDecodeError, UnicodeEncodeError) as err:
            queued_message.defer()
            logger.warning("Message to %s deferred due to failure: %s" %
                           (message.to_address.encode("utf-8"), err))
            try:
                log_message = unicode(err)
            except NameError:
                log_message = err
            result = constants.RESULT_FAILED

    if log:
        models.Log.objects.create(message=message, result=result,
                                  log_message=log_message)

    if opened_connection:
        smtp_connection.close()
    return result


def send_message(email_message, smtp_connection=None):
    """
    Send an EmailMessage, returning a response code as to the action taken.

    The response codes can be found in ``django_yubin.constants``. The
    response will be either ``RESULT_FAILED`` for a failed send,
    ``RESULT_SENT`` for a successfully sent message or ``RESULT_SKIPPED`` if
    ``settings.PAUSE_SEND`` is True.

    To allow optimizations if multiple messages are to be sent, an SMTP
    connection can be provided. Otherwise an SMTP connection will be opened
    to send this message.

    This function does not perform any queueing.

    """
    if settings.PAUSE_SEND:
        logger.warning("Sending is paused, exiting without sending message.")
        return constants.RESULT_SKIPPED

    if smtp_connection is None:
        smtp_connection = get_connection()
    opened_connection = False

    try:
        opened_connection = smtp_connection.open()
        try:
            smtp_connection.connection.sendmail(
                email_message.from_email,
                email_message.recipients(),
                smart_str(email_message.message().as_string()).encode('utf-8'))
        except UnicodeDecodeError:
            message = email_message.message()
            charset = message.get_charset()
            if charset:
                charset = charset.get_output_charset()
            else:
                charset = 'utf-8'
            smtp_connection.connection.sendmail(
                email_message.from_email,
                email_message.recipients(),
                force_bytes(message.as_string(), charset))
        result = constants.RESULT_SENT
    except (SocketError, smtplib.SMTPSenderRefused,
            smtplib.SMTPRecipientsRefused, smtplib.SMTPAuthenticationError,
            UnicodeDecodeError, UnicodeEncodeError) as err:
        result = constants.RESULT_FAILED
        logger.warning("Message from %s failed due to: %s" %
                       (email_message.from_email, err))

    if opened_connection:
        smtp_connection.close()
    return result



# Generated by Django 2.0.6 on 2018-11-28 10:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0004_message_date_sent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='queuedmessage',
            name='priority',
            field=models.PositiveSmallIntegerField(choices=[(0, 'now'), (1, 'high'), (3, 'normal'), (5, 'low')], default=3),
        ),
    ]



#!/usr/bin/env python
# encoding: utf-8
# ----------------------------------------------------------------------------

# Deprecated, keeped here for backward compatibility, use PRIORITY_NOW instead.
PRIORITY_EMAIL_NOW = 0

PRIORITY_NOW_NOT_QUEUED = -1
PRIORITY_NOW = 0
PRIORITY_HIGH = 1
PRIORITY_NORMAL = 3
PRIORITY_LOW = 5

RESULT_SENT = 0
RESULT_SKIPPED = 1
RESULT_FAILED = 2

PRIORITIES = {
    'now-not-queued': PRIORITY_NOW_NOT_QUEUED,
    'now': PRIORITY_NOW,
    'high': PRIORITY_HIGH,
    'normal': PRIORITY_NORMAL,
    'low': PRIORITY_LOW,
}

PRIORITY_HEADER = 'X-Mail-Queue-Priority'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2017-04-05 14:35
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='queuedmessage',
            name='priority',
            field=models.PositiveSmallIntegerField(choices=[(1, 'high'), (3, 'normal'), (5, 'low'), (0, 'now')], default=3),
        ),
    ]



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import os

from django.core.management.base import BaseCommand

from ...messages import BasicHTMLAttachmentEmailMessageView


class Command(BaseCommand):
    help = 'Create fake mails for testing.'

    def add_arguments(self, parser):
        parser.add_argument(
            '-q',
            '--quantity',
            dest='quantity',
            default=1,
            help='Number of fake mails to create.',
        )

    def handle(self, *args, **options):
        number = int(options['quantity'])

        from_email = 'test@example.com'
        to = ['recipient@example.com']

        for i in range(1, number + 1):
            subject = 'Subject test %s ✉️ 🙂 àäá' % i
            content = 'Body test <strong>%s</strong> ✉️ 🙂 àäá.' % i
            attachment_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'sample.pdf')
            with open(attachment_path, 'rb') as f:
                attachment = f.read()
            filename = 'sample.pdf'
            mimetype = 'application/pdf'
            message = BasicHTMLAttachmentEmailMessageView(subject, content, attachment, filename, mimetype)
            message.send(from_email=from_email, to=to)

        # This output is checked in tests.
        self.stdout.write('Created email(s): %d' % number)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-16 15:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Blacklist',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=200)),
                ('date_added', models.DateTimeField(default=django.utils.timezone.now)),
            ],
            options={
                'verbose_name_plural': 'blacklisted e-mail addresses',
                'ordering': ('-date_added',),
                'verbose_name': 'blacklisted e-mail address',
            },
        ),
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('result', models.PositiveSmallIntegerField(choices=[(0, 'success'), (1, 'not sent (blacklisted)'), (2, 'failure')])),
                ('date', models.DateTimeField(default=django.utils.timezone.now)),
                ('log_message', models.TextField()),
            ],
            options={
                'ordering': ('-date',),
            },
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('to_address', models.CharField(max_length=200)),
                ('from_address', models.CharField(max_length=200)),
                ('subject', models.CharField(max_length=255)),
                ('encoded_message', models.TextField()),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now)),
            ],
            options={
                'ordering': ('date_created',),
            },
        ),
        migrations.CreateModel(
            name='QueuedMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('priority', models.PositiveSmallIntegerField(choices=[(1, 'high'), (3, 'normal'), (5, 'low')], default=3)),
                ('deferred', models.DateTimeField(blank=True, null=True)),
                ('retries', models.PositiveIntegerField(default=0)),
                ('date_queued', models.DateTimeField(default=django.utils.timezone.now)),
                ('message', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='django_yubin.Message')),
            ],
            options={
                'ordering': ('priority', 'date_queued'),
            },
        ),
        migrations.AddField(
            model_name='log',
            name='message',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to='django_yubin.Message'),
        ),
    ]



# Generated by Django 2.0.6 on 2018-06-27 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0003_auto_20171201_1506'),
    ]

    operations = [
        migrations.AddField(
            model_name='message',
            name='date_sent',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-12-01 14:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_yubin', '0002_auto_20170405_1635'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='result',
            field=models.PositiveSmallIntegerField(choices=[(0, 'success'), (1, 'not sent (blacklisted or paused)'), (2, 'failure')]),
        ),
    ]



#!/usr/bin/env python
# encoding: utf-8
# ----------------------------------------------------------------------------

from django.conf import settings
from django_yubin import constants

# Provide a way of temporarily pausing the sending of mail.
PAUSE_SEND = getattr(settings, "MAILER_PAUSE_SEND", False)

USE_BACKEND = getattr(settings, 'MAILER_USE_BACKEND',
                      'django.core.mail.backends.smtp.EmailBackend')

# Default priorities for the mail_admins and mail_managers methods.
MAIL_ADMINS_PRIORITY = getattr(settings, 'MAILER_MAIL_ADMINS_PRIORITY',
                               constants.PRIORITY_HIGH)
MAIL_MANAGERS_PRIORITY = getattr(settings, 'MAILER_MAIL_MANAGERS_PRIORITY',
                                 None)

# When queue is empty, how long to wait (in seconds) before checking again.
EMPTY_QUEUE_SLEEP = getattr(settings, "MAILER_EMPTY_QUEUE_SLEEP", 30)

# Lock timeout value. how long to wait for the lock to become available.
# default behavior is to never wait for the lock to be available.
LOCK_WAIT_TIMEOUT = max(getattr(settings, "MAILER_LOCK_WAIT_TIMEOUT", 0), 0)

# An optional alternate lock path, potentially useful if you have multiple
# projects running on the same server.
LOCK_PATH = getattr(settings, "MAILER_LOCK_PATH", None)

# When MAILER_TEST_MODE is True, recipient addresses of all messages are replaced with
# the email addresses set in MAILER_TEST_EMAIL before being sent
MAILER_TEST_MODE = getattr(settings, "MAILER_TEST_MODE", False)
MAILER_TEST_EMAIL = getattr(settings, "MAILER_TEST_EMAIL", '')

MAILER_HC_QUEUED_LIMIT_OLD = getattr(settings, "MAILER_HC_QUEUED_LIMIT_OLD", 30)



#!/usr/bin/env python
# encoding: utf-8
# ----------------------------------------------------------------------------

from django.db import models
from django_yubin import constants

try:
    from django.utils.timezone import now
except ImportError:
    import datetime
    now = datetime.datetime.now


class QueueMethods(object):
    """
    A mixin which provides extra methods to a QuerySet/Manager subclass.

    """

    def exclude_future(self):
        """
        Exclude future time-delayed messages.

        """
        return self.exclude(date_queued__gt=now())

    def now_priority(self):
        """
        Return a QuerySet of now priority queued messages.

        """
        return self.filter(priority=constants.PRIORITY_NOW)

    def high_priority(self):
        """
        Return a QuerySet of high priority queued messages.

        """
        return self.filter(priority=constants.PRIORITY_HIGH)

    def normal_priority(self):
        """
        Return a QuerySet of normal priority queued messages.

        """
        return self.filter(priority=constants.PRIORITY_NORMAL)

    def low_priority(self):
        """
        Return a QuerySet of low priority queued messages.

        """
        return self.filter(priority=constants.PRIORITY_LOW)

    def non_deferred(self):
        """
        Return a QuerySet containing all non-deferred queued messages,
        excluding "future" messages.

        """
        return self.exclude_future().filter(deferred=None)

    def deferred(self):
        """
        Return a QuerySet of all deferred messages in the queue, excluding
        "future" messages.

        """
        return self.exclude_future().exclude(deferred=None)


class QueueQuerySet(QueueMethods, models.query.QuerySet):
    pass


class QueueManager(QueueMethods, models.Manager):
    use_for_related_fields = True

    def get_queryset(self):
        return QueueQuerySet(self.model, using=self._db)

    def retry_deferred(self, max_retries=None, new_priority=None):
        """
        Reset the deferred flag for all deferred messages so they will be
        retried.

        If ``max_retries`` is set, deferred messages which have been retried
        more than this many times will *not* have their deferred flag reset.

        If ``new_priority`` is ``None`` (default), deferred messages retain
        their original priority level. Otherwise all reset deferred messages
        will be set to this priority level.

        """
        queryset = self.deferred()
        if max_retries:
            queryset = queryset.filter(retries__lte=max_retries)
        count = queryset.count()
        update_kwargs = dict(deferred=None, retries=models.F('retries')+1)
        if new_priority is not None:
            update_kwargs['priority'] = new_priority
        queryset.update(**update_kwargs)
        return count



#!/usr/bin/env python
# encoding: utf-8
# ----------------------------------------------------------------------------

from django.conf.urls import url
from django.contrib import admin

try:
    # from django 1.10 and above
    from django.urls import reverse
except ImportError:
    # until django 1.9
    from django.core.urlresolvers import reverse
from django.db import IntegrityError
from django.http import HttpResponse
from django.shortcuts import render
from django.utils.safestring import mark_safe

from django_yubin import models

from .mail_utils import get_attachments, get_attachment


class Message(admin.ModelAdmin):
    def message_link(self, obj):
        url = reverse('admin:mail_detail', args=(obj.id,))
        return mark_safe("""<a href="%s" onclick="return showAddAnotherPopup(this);">show</a>""" % url)

    message_link.allow_tags = True
    message_link.short_description = u'Show'

    list_display = ('from_address', 'to_address', 'subject', 'date_created', 'date_sent', 'message_link')
    list_filter = ('date_created', 'date_sent')
    search_fields = ('to_address', 'subject', 'from_address',
                     'encoded_message',)
    date_hierarchy = 'date_created'
    ordering = ('-date_created',)
    actions = ['re_send', ]

    def re_send(self, request, queryset):
        """
        Re sends a previus sent e-mail. The messages shouldn't be in the queue and
        is put in the NORMAL priority
        """
        messages_sent = 0
        for selected_mail in queryset:
            qm = models.QueuedMessage(message=selected_mail)
            try:
                qm.save()
                messages_sent += 1
            except IntegrityError:
                self.message_user(request, 'Message %s is already in the queue' % selected_mail.id)
        self.message_user(request, "%s messages had been re-sent" % messages_sent)
    re_send.short_description = 're-send selected emails'

    def get_urls(self):
        urls = super(Message, self).get_urls()
        custom_urls = [
            url(r'^mail/(?P<pk>\d+)/$',
                self.admin_site.admin_view(self.detail_view),
                name='mail_detail'),
            url('^mail/attachment/(?P<pk>\d+)/(?P<firma>[0-9a-f]{32})/$',
                self.admin_site.admin_view(self.download_view),
                name="mail_download"),
            url('^mail/html/(?P<pk>\d+)/$',
                self.admin_site.admin_view(self.html_view),
                name="mail_html"),
        ]
        return custom_urls + urls

    def detail_view(self, request, pk):
        instance = models.Message.objects.get(pk=pk)
        msg = instance.get_pyz_message()
        context = {'subject': msg.get_subject(), 'from': msg.get_address('from'), 'to': msg.get_addresses('to'),
                   'cc': msg.get_addresses('cc'),
                   'msg_text': msg.text_part.part.get_payload(
                       decode=self.is_base64(msg, 'text_part')) if msg.text_part else None,
                   'msg_html': msg.html_part.part.get_payload(
                       decode=self.is_base64(msg, 'html_part')) if msg.html_part else None,
                   'attachments': get_attachments(msg), 'is_popup': True, 'object': instance}
        return render(request, 'django_yubin/message_detail.html', context)

    def download_view(self, request, pk, firma):
        instance = models.Message.objects.get(pk=pk)
        msg = instance.get_pyz_message()
        arx = get_attachment(msg, key=firma)
        response = HttpResponse(content_type=arx.tipus)
        response['Content-Disposition'] = 'filename=' + arx.filename
        response.write(arx.payload)
        return response

    def html_view(self, request, pk):
        instance = models.Message.objects.get(pk=pk)
        msg = instance.get_pyz_message()
        msg.html_part.part._charset = 'utf-8'
        context = {'msg_html': msg.html_part.part.get_payload(decode=self.is_base64(msg, 'html_part'))}
        return render(request, 'django_yubin/html_detail.html', context)

    @staticmethod
    def is_base64(msg, part='html_part'):
        """
        detect whether is a base64 encoding or not

        :param msg, part:
        :return:
        """
        if part == 'html_part':
            return any(it[1] == 'base64' for it in msg.html_part.part._headers)
        elif part == 'text_part':
            return any(it[1] == 'base64' for it in msg.text_part.part._headers)
        return False


class MessageRelatedModelAdmin(admin.ModelAdmin):
    list_select_related = True

    def message__to_address(self, obj):
        return obj.message.to_address

    message__to_address.admin_order_field = 'message__to_address'

    def message__from_address(self, obj):
        return obj.message.from_address

    message__from_address.admin_order_field = 'message__from_address'

    def message__subject(self, obj):
        return obj.message.subject

    message__subject.admin_order_field = 'message__subject'

    def message__date_created(self, obj):
        return obj.message.date_created

    message__date_created.admin_order_field = 'message__date_created'


class QueuedMessage(MessageRelatedModelAdmin):
    def not_deferred(self, obj):
        return not obj.deferred

    not_deferred.boolean = True
    not_deferred.admin_order_field = 'deferred'

    def message_link(self, obj):
        url = reverse('admin:mail_detail', args=(obj.message.id,))
        return mark_safe("""<a href="%s" onclick="return showAddAnotherPopup(this);">%s</a>""" % (url, obj.message))

    message_link.allow_tags = True
    message_link.short_description = u'Message'

    list_display = ('id', 'message_link', 'message__to_address',
                    'message__from_address', 'message__subject',
                    'message__date_created', 'priority', 'not_deferred')
    list_filter = ('priority', 'deferred')


class Blacklist(admin.ModelAdmin):
    list_display = ('email', 'date_added')


class Log(MessageRelatedModelAdmin):
    def message_link(self, obj):
        url = reverse('admin:mail_detail', args=(obj.message.id,))
        return mark_safe("""<a href="%s" onclick="return showAddAnotherPopup(this);">show</a>""" % url)

    message_link.allow_tags = True
    message_link.short_description = u'Message'

    list_display = ('id', 'result', 'message__to_address', 'message__subject',
                    'date', 'message_link')
    list_filter = ('result',)
    list_display_links = ('id', 'result')


admin.site.register(models.Message, Message)
admin.site.register(models.QueuedMessage, QueuedMessage)
admin.site.register(models.Blacklist, Blacklist)
admin.site.register(models.Log, Log)



import itertools


def peek(sequence):
    """
    Returns the value of the top of the sequence without removing the value
    from the data.
    """
    iterable = iter(sequence)
    try:
        first = next(iterable)
        return first, itertools.chain([first], iterable)
    except StopIteration:
        return None, []



#!/usr/bin/env python
# encoding: utf-8
# ----------------------------------------------------------------------------

import hashlib


def unimplemented(*args, **kwargs):
    raise NotImplementedError


class Attachment(object):
    """
    Utility class to contain the attachment information
    """

    def __init__(self, mailpart):
        self.filename = mailpart.sanitized_filename
        self.tipus = mailpart.type
        self.charset = mailpart.charset
        self.content_description = mailpart.part.get('Content-Description')
        self.payload = mailpart.get_payload()
        self.length = len(self.payload)
        self.firma = hashlib.md5(self.payload).hexdigest()


def get_attachments(msg):
    """
    Returns a list with all the mail attachments
    """

    attachments = []
    for mailpart in msg.mailparts:
        if not mailpart.is_body and mailpart.disposition == 'attachment':
            attachment = Attachment(mailpart)
            attachments.append(attachment)
    return attachments


def get_attachment(msg, key):
    """
    Given a msg returns the attachment who's signature (md5 key) matches
    the key value
    """

    for mailpart in msg.mailparts:
        if not mailpart.is_body and mailpart.disposition == 'attachment':
            attachment = Attachment(mailpart)
            if attachment.firma == key:
                return attachment
    return None



import logging


LOGGING_LEVEL = {'0': logging.ERROR, '1': logging.WARNING, '2': logging.DEBUG, '3': logging.DEBUG}


def create_handler(verbosity, message='%(message)s'):
    """
    Create a handler which can output logged messages to the console (the log
    level output depends on the verbosity level).
    """
    handler = logging.StreamHandler()
    handler.setLevel(LOGGING_LEVEL[str(verbosity)])
    formatter = logging.Formatter(message)
    handler.setFormatter(formatter)
    return handler



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

from ...messages import BasicHTMLEmailMessageView


class Command(BaseCommand):
    help = 'Send a simple email in order to check connection parameters.'

    def add_arguments(self, parser):
        parser.add_argument(
            '-s',
            '--subject',
            dest='subject',
            default='Hello World',
            help='Email subject.',
        )

        parser.add_argument(
            '-c',
            '--content',
            dest='content',
            default='Hello World',
            help='Email content.',
        )

        parser.add_argument(
            '-t',
            '--to',
            dest='to',
            default='',
            help='Email address to send message.',
        )

    def handle(self, *args, **options):
        from_email = settings.DEFAULT_FROM_EMAIL
        to = options['to']
        if to:
            to_list = [to]
        else:
            to_list = [x[1] for x in settings.ADMINS if len(x) > 0]

        if not to_list:
            raise CommandError('Please provide an email address or set a valid settings.ADMINS configuration')

        subject = options['subject']
        content = options['content']
        message = BasicHTMLEmailMessageView(subject, content)
        message.send(from_email=from_email, to=to_list)

        # This output is checked in tests.
        self.stdout.write('Created email(s): 1')



from django.http import HttpResponse
from django.views.generic import View
from django.utils import timezone

from django_yubin.models import QueuedMessage
from django_yubin import settings


class MailHealthCheckView(View):
    def get(self, request, *args, **kwargs):
        oldest, too_old = 0, False
        message = QueuedMessage.objects.order_by('date_queued').first()

        if message:
            # In minutes...
            total_seconds = (timezone.now().astimezone() - message.date_queued.astimezone()).total_seconds()
            oldest = round(total_seconds / 60)
            too_old = oldest > settings.MAILER_HC_QUEUED_LIMIT_OLD

        response = 'oldest_queued_email: {} mins\nemails_queued_too_old: {}'.format(oldest, ('no', 'yes')[too_old])
        return HttpResponse(response, content_type='text/plain')





# encoding: utf-8

import datetime
import logging

from django.core.management.base import BaseCommand

from django_yubin.management.commands import create_handler
from django_yubin.models import Message


class Command(BaseCommand):
    help = 'Delete the mails created before -d days (default 90)'

    def add_arguments(self, parser):
        parser.add_argument(
            '-d',
            '--days',
            dest='days',
            type=int,
            default=90,
            help="Cleanup mails older than this many days, defaults to 90.",
        )

    def handle(self, verbosity, **options):
        # Delete mails and their related logs and queued created before X days
        logger = logging.getLogger('django_yubin')
        handler = create_handler(verbosity)
        logger.addHandler(handler)

        today = datetime.date.today()
        cutoff_date = today - datetime.timedelta(options['days'])
        count = Message.objects.filter(date_created__lt=cutoff_date).count()
        if count:
            Message.objects.filter(date_created__lt=cutoff_date).delete()
            logger.info("Deleted %s mails created before %s " %
                        (count, cutoff_date))



# encoding: utf-8

import sys

from django.core.management.base import BaseCommand
from django.db import connection
from django_yubin import models, settings
from django_yubin.engine import send_all


class Command(BaseCommand):
    help = 'Iterate the mail queue, attempting to send all mail.'

    def add_arguments(self, parser):
        parser.add_argument(
            '-b',
            '--block_size',
            dest='block_size',
            type=int,
            default=500,
            help='The number of messages to iterate before checking the queue '
                 'again (in case new messages have been added while the queue '
                 'is being cleared).',
        )
        parser.add_argument(
            '-l',
            '--message_limit',
            dest='message_limit',
            type=int,
            default=0,
            help='The maximum number of messages to send from the queue in a '
                 'single pass (to avoid email rate throttles).',
        )
        parser.add_argument(
            '-c',
            '--count',
            dest='count',
            action='store_true',
            default=False,
            help='Return the number of messages in the queue (without '
                 'actually sending any)',
        )

    def handle(self, verbosity, **options):
        # If this is just a count request the just calculate, report and exit.
        if options['count']:
            queued = models.QueuedMessage.objects.non_deferred().count()
            deferred = models.QueuedMessage.objects.deferred().count()
            sys.stdout.write('%s queued message%s (and %s deferred message%s).'
                             '\n' % (queued, queued != 1 and 's' or '',
                                     deferred, deferred != 1 and 's' or ''))
            sys.exit()

        send_all(options['block_size'],
                 backend=settings.USE_BACKEND,
                 message_limit=options['message_limit'])

        # Stop superfluous "unexpected EOF on client connection" errors in
        # Postgres log files caused by the database connection not being
        # explicitly closed.
        connection.close()



#!/usr/bin/env python
# encoding: utf-8
# ----------------------------------------------------------------------------

from setuptools import setup


INSTALL_REQUIRES = [
    'pyzmail;python_version<"3.6"',
    'pyzmail36;python_version>="3.6"',
    'lockfile',
    'pytz',
]

TEST_REQUIREMENTS = [
    'six'
]


with open('docs/index.rst') as docs_index:
    long_description = docs_index.read()


setup(
    name='django-yubin',
    version='1.3.1',
    description=("A reusable Django app for composing and queueing emails "
                 "Adds django-mailer2 + django-mailviews + others"),
    long_description=long_description,
    author='Antoni Aloy',
    author_email='aaloy@apsl.net',
    url='http://github.com/APSL/django-yubin',
    install_requires=INSTALL_REQUIRES,
    packages=[
        'django_yubin',
        'django_yubin.management',
        'django_yubin.management.commands',
        'django_yubin.migrations',
    ],
    include_package_data=True,
    # tests
    tests_require=TEST_REQUIREMENTS,
    test_suite='runtests.runtests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
    ]
)



# encoding: utf-8

"""
This command returns the status of the queue in the format:
    queued/deferred/seconds

where:
    queued      is the number of queued messages we have actually
    deferred    number of deferred messages we have actually
    seconds     age in seconds of the oldest messages

Example:

    2/0/10

means we have 2 queued messages, 0 defered messaged and than the oldest message
in the queue is just 2 seconds old.
"""
from __future__ import unicode_literals

from django.core.management.base import BaseCommand
from django_yubin.models import QueuedMessage
from django.utils.timezone import now


class Command(BaseCommand):
    help = "Returns a string with the queue status as queued/deferred/seconds"

    def handle(self, *args, **kwargs):
        # If this is just a count request the just calculate, report and exit.
        queued = QueuedMessage.objects.non_deferred().count()
        deferred = QueuedMessage.objects.deferred().count()
        try:
            oldest = QueuedMessage.objects.non_deferred().order_by(
                'date_queued')[0]
            seconds = (now() - oldest.date_queued).seconds
        except (IndexError, QueuedMessage.DoesNotExist):
            seconds = 0

        # This output is checked in tests.
        self.stdout.write('%s/%s/%s\n' % (queued, deferred, seconds))
